import { clearNotify } from './entity';
import { Query, hasAll } from './query';
var EntityManager = /** @class */ (function () {
    function EntityManager() {
        var _this = this;
        this.changed = [];
        this.removed = [];
        this.queries = {
            '': new Query(function () { return true; }, []),
        };
        this.query = function () {
            var components = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                components[_i] = arguments[_i];
            }
            var queryId = getQueryId(components);
            if (!_this.queries[queryId]) {
                _this.queries[queryId] = new Query(hasAll(components), _this.queries[''].entities);
            }
            return _this.queries[queryId].entities;
        };
        this.scheduleUpdate = function (entity) { return _this.changed.push(entity); };
        this.scheduleRemove = function (entity) { return _this.removed.push(entity); };
    }
    EntityManager.prototype.processUpdates = function () {
        var _loop_1 = function (query) {
            this_1.changed.forEach(function (entity) { return query.onChange(entity); });
            this_1.removed.forEach(function (entity) { return query.onRemove(entity); });
        };
        var this_1 = this;
        for (var _i = 0, _a = Object.values(this.queries); _i < _a.length; _i++) {
            var query = _a[_i];
            _loop_1(query);
        }
        this.changed.forEach(clearNotify);
        this.changed.length = 0;
        this.removed.forEach(clearNotify);
        this.removed.length = 0;
    };
    return EntityManager;
}());
export { EntityManager };
function getQueryId(components) {
    return components.map(function (component) { return component.type; }).sort().join('+');
}
//# sourceMappingURL=entity-manager.js.map