var Entity = /** @class */ (function () {
    function Entity(onChange) {
        var _this = this;
        this.components = {};
        this.didNotify = false;
        this.notify = function () {
            if (!_this.didNotify) {
                _this.didNotify = true;
                onChange(_this);
            }
        };
    }
    Entity.prototype.add = function (component) {
        if (component == null) {
            throw new TypeError('Argument is not a component instance.');
        }
        var componentClass = component.constructor;
        if (!isComponentClass(componentClass)) {
            throw new TypeError('Argument is not a component instance.');
        }
        else if (this.has(componentClass)) {
            throw new Error('Component type already present.');
        }
        this.components[componentClass.type] = component;
        this.notify();
        return this;
    };
    Entity.prototype.has = function (componentClass) {
        if (!isComponentClass(componentClass)) {
            throw new TypeError('Argument is not a component class.');
        }
        return this.components.hasOwnProperty(componentClass.type);
    };
    Entity.prototype.get = function (componentClass) {
        if (typeof componentClass !== 'function') {
            throw new TypeError('Argument is not a component class.');
        }
        var component = this.components[componentClass.type];
        if (!component) {
            throw new TypeError('Component type not present.');
        }
        return component;
    };
    Entity.prototype.remove = function (componentClass) {
        if (typeof componentClass !== 'function') {
            throw new TypeError('Argument is not a component class.');
        }
        delete this.components[componentClass.type];
        this.notify();
        return this;
    };
    return Entity;
}());
export { Entity };
function isComponentClass(value) {
    return typeof value === 'function' && typeof value.type === 'string';
}
export function clearNotify(entity) {
    entity.didNotify = false;
}
//# sourceMappingURL=entity.js.map